#include "commands.h"

#include "gui_help.h"
#include "gui.h"

#include <stdio.h>
#include <string.h>
#include <gtk/gtk.h>

static const char *help[] = {
#include "help.h"
0
};

static gui_help h;

static void show_help(void)
{
  int i;
  GtkTextBuffer *b;

  if (h.shown) return;

  init_gui_help(&h);
  h.shown = 1;
  gtk_widget_show_all(h.w);

  b = gtk_text_view_get_buffer(GTK_TEXT_VIEW(h.t));

  gtk_text_buffer_insert_at_cursor(b, help[0], strlen(help[0]));

  for (i = 1; help[i]; i++) {
    gtk_text_buffer_insert_at_cursor(b, "\n", strlen("\n"));
    gtk_text_buffer_insert_at_cursor(b, help[i], strlen(help[i]));
  }
}

void do_command(mainwin *this, const char *c)
{
  float f;
  int i;

  if (c[0] && (c[1] == ' ' || c[1] == 0)) switch (c[0]) {
  case 'f':
    sscanf(c+1, "%g", &f);
    gui_set_min_freq(this, f);
    printf("min freq set to %g\n", this->min_freq);
    break;
  case 'F':
    sscanf(c+1, "%g", &f);
    gui_set_max_freq(this, f);
    printf("max freq set to %g\n", this->max_freq);
    break;
  case 'd':
    sscanf(c+1, "%g", &f);
    gui_colors_set_mindB(this, f);
    printf("dB range is: %g - %g\n",
           this->colormap.mindB, this->colormap.maxdB);
    break;
  case 'D':
    sscanf(c+1, "%g", &f);
    gui_colors_set_maxdB(this, f);
    printf("dB range is: %g - %g\n",
           this->colormap.mindB, this->colormap.maxdB);
    break;
  case 'l':
    sscanf(c+1, "%d", &i);
    power_buffer_set_to_process(&this->f, i);
    break;
  case 'p':
    gui_pause_unpause(this);
    if (this->jack->paused)
      printf("pause\n");
    else
      printf("run\n");
    break;
  case 'o':
    sscanf(c+1, "%g", &f);
    power_buffer_set_overlap(&this->f, f);
    break;
  case 'q': gui_quit(); return;
  case 'h': goto show_help;
  case 's':
    sscanf(c+1, "%d", &i);
    if (!(i == 128 || i == 256 ||
          i == 512 || i == 1024 || i == 2048 || i == 4096 || i == 8192 ||
          i == 16384 || i == 32768 || i == 65536 || i == 131072 ||
          i == 262144 || i == 524288)) {
      printf("bad fft size %d\n", i);
      break;
    }
    gui_fft_size(this, i);
    break;
  case 'c':
    gui_load_colormap(this, c+2);
    break;
  case 'w':
    fft_window_type_from_string(&this->f.window, c+2);
    break;
  default: goto long_command;
  } else goto long_command;
done:
  gui_state(this);
  return;
long_command:
  if (!strncmp(c, "help", 4)) {
show_help:
    show_help();
    for (i = 0; help[i]; i++)
      printf("%s\n", help[i]);
  } else if (!strcmp(c, "cs")) {
    gui_show_colormap_selection(this);
  } else if (!strcmp(c, "nr")) {
    this->redraw_lines = redraw_lines_nearest;
    this->redraw_lines(this, 0, this->width-1);
  } else if (!strcmp(c, "av")) {
    this->redraw_lines = redraw_lines_average;
    this->redraw_lines(this, 0, this->width-1);
  } else if (strlen(c)) {
    printf("unknwon command '%s'\n", c);
  }
  goto done;
}
