#include "gui.h"
#include "defs.h"
#include "colors.h"

#include <stdlib.h>
#include <stdio.h>
#include <math.h>

void redraw_lines_colors(mainwin *this)
{
  int i, j;
  for (i = 0; i < this->col_height; i++) {
    float dB = R3(i, 0, this->col_height-1,
                  this->colormap.maxdB, this->colormap.mindB);
    unsigned char *rgb = get_color(&this->colormap, dB);
    for (j = 0; j < 10; j++) {
      this->colors_image[(i*10+j)*3] = rgb[0];
      this->colors_image[(i*10+j)*3+1] = rgb[1];
      this->colors_image[(i*10+j)*3+2] = rgb[2];
    }
  }
  gui_force_redraw_colors(this);
}

void redraw_lines_average(mainwin *this, int first, int last)
{
  power_buffer *b = &this->f;
  float *buf;
  unsigned char *rgb;
  int i, j;
  float f0 = R3(0, 0, this->height, this->max_freq, this->min_freq);
  float pos0 = R3(f0, 0, this->jack->sample_rate, 0, this->f.fft.n);
  float f1 = R3(1, 0, this->height, this->max_freq, this->min_freq);
  float pos1 = R3(f1, 0, this->jack->sample_rate, 0, this->f.fft.n);
  float pixel_size = pos0 - pos1;
  for (i = first; i <= last; i++) {
    buf = power_buffer_get_line(b, i);
    for (j = 0; j < this->height; j++) {
      float f = R3(j, 0, this->height, this->max_freq, this->min_freq);
      float pos_end = R3(f, 0, this->jack->sample_rate, 0, this->f.fft.n);
      float pos_start = pos_end - pixel_size;
      int pstart, pend;
      float power = 0;
      int ii;
      /* this if is necessary, float precision I guess... */
      if (pos_start < 0) pos_start = 0;
      /* center */
      pstart = ceil(pos_start);
      pend = floor(pos_end);
      for (ii = pstart; ii < pend; ii++) power += buf[ii];
      /* pre/post */
      if (ceil(pos_end) == pstart) {
        /* all inside one cell */
        power = buf[pend] * pixel_size;
      } else {
        /* some room before and/or after */
        float wstart = pstart - pos_start;
        float wend = pos_end - pend;
        if (wstart) power += buf[pstart-1] * wstart;
        if (wend) power += buf[pend] * wend;
      }
      power /= pixel_size;
      rgb = get_color(&this->colormap, power);
      this->image[(j * this->width + i) * 3] = rgb[0];
      this->image[(j * this->width + i) * 3 + 1] = rgb[1];
      this->image[(j * this->width + i) * 3 + 2] = rgb[2];
    }
  }
  gui_force_redraw(this);
}

void redraw_lines_nearest(mainwin *this, int first, int last)
{
  power_buffer *b = &this->f;
  float *buf;
  unsigned char *rgb;
  int i, j;
  for (i = first; i <= last; i++) {
    buf = power_buffer_get_line(b, i);
    for (j = 0; j < this->height; j++) {
      /* nearest */
      float f = R3(j, 0, this->height-1, this->max_freq, this->min_freq);
      int pos = R3(f, 0, this->jack->sample_rate, 0, this->f.fft.n);
      if (pos < 0 || pos > this->f.fft.n/2+1) abort();
      float power = buf[pos];
      rgb = get_color(&this->colormap, power);
      this->image[(j * this->width + i) * 3] = rgb[0];
      this->image[(j * this->width + i) * 3 + 1] = rgb[1];
      this->image[(j * this->width + i) * 3 + 2] = rgb[2];
    }
  }
  gui_force_redraw(this);
}

static int _gui_set_min_freq(mainwin *this, float f)
{
  if (f < 0) f = 0;
  if (f > this->jack->sample_rate / 2 - 1) f = this->jack->sample_rate / 2 - 1;
  if (f >= this->max_freq) f = this->max_freq - 1;
  if (this->min_freq == f) return 0;
  this->min_freq = f;
  return 1;
}

void gui_set_min_freq(mainwin *this, float f)
{
  if (_gui_set_min_freq(this, f))
    this->redraw_lines(this, 0, this->width-1);
}

static int _gui_set_max_freq(mainwin *this, float f)
{
  if (f > this->jack->sample_rate / 2) f = this->jack->sample_rate / 2;
  if (f < 1) f = 1;
  if (f <= this->min_freq) f = this->min_freq + 1;
  if (this->max_freq == f) return 0;
  this->max_freq = f;
  return 1;
}

void gui_set_max_freq(mainwin *this, float f)
{
  if (_gui_set_max_freq(this, f))
    this->redraw_lines(this, 0, this->width-1);
}

void gui_colors_set_mindB(mainwin *this, float f)
{
  if (f > this->colormap.maxdB - 0.1) f = this->colormap.maxdB - 0.1;
  if (this->colormap.mindB == f) return;
  this->colormap.mindB = f;
  this->redraw_lines(this, 0, this->width-1);
}

void gui_colors_set_maxdB(mainwin *this, float f)
{
  if (f < this->colormap.mindB + 0.1) f = this->colormap.mindB + 0.1;
  if (this->colormap.maxdB == f) return;
  this->colormap.maxdB = f;
  this->redraw_lines(this, 0, this->width-1);
}

#define MOUSE_MIN_FREQ 1
#define MOUSE_MAX_FREQ 2
#define MOUSE_SCROLL_FREQ 3
#define MOUSE_SPEED 4
#define MOUSE_MINDB 5
#define MOUSE_MAXDB 6

void gui_mouse_click(struct mainwin *w, int x, int y,
                     int has_shift, int has_control)
{
  if (has_shift) {
    if (y > w->height / 2)
      w->mouse_state = MOUSE_MIN_FREQ;
    else
      w->mouse_state = MOUSE_MAX_FREQ;
  }
  else if (has_control)
    w->mouse_state = MOUSE_SPEED;
  else w->mouse_state = MOUSE_SCROLL_FREQ;
  w->mouse_x = x;
  w->mouse_y = y;
  w->mouse_min_freq = w->min_freq;
  w->mouse_max_freq = w->max_freq;
  w->mouse_to_process = w->f.to_process;
}

void gui_mouse_move(struct mainwin *w, int x, int y)
{
  float delta_freq = (y - w->mouse_y) *
                     (w->mouse_max_freq - w->mouse_min_freq) / 200;
#if 0
                     (w->mouse_max_freq - w->mouse_min_freq) / w->height;
#endif
  if (w->mouse_state == MOUSE_MIN_FREQ) {
    gui_set_min_freq(w, w->mouse_min_freq + delta_freq);
    gui_state(w);
  } else if (w->mouse_state == MOUSE_MAX_FREQ) {
    gui_set_max_freq(w, w->mouse_max_freq + delta_freq);
    gui_state(w);
  } else if (w->mouse_state == MOUSE_SCROLL_FREQ) {
    if (delta_freq < 0) {
      float old_min = w->min_freq;
      _gui_set_min_freq(w, w->mouse_min_freq + delta_freq);
      delta_freq = w->min_freq - old_min;
      gui_set_max_freq(w, w->max_freq + delta_freq);
    } else {
      float old_max = w->max_freq;
      _gui_set_max_freq(w, w->mouse_max_freq + delta_freq);
      delta_freq = w->max_freq - old_max;
      gui_set_min_freq(w, w->min_freq + delta_freq);
    }
    gui_state(w);
  } else if (w->mouse_state == MOUSE_SPEED) {
    delta_freq = pow(2, (x - w->mouse_x) / 400.);
    power_buffer_set_to_process(&w->f, w->mouse_to_process * delta_freq);
    gui_state(w);
  }
}

void gui_mouse_unclick(struct mainwin *w)
{
  w->mouse_state = 0;
}

void gui_colors_mouse_click(struct mainwin *w, int x, int y)
{
  if (y > w->col_height / 2)
    w->mouse_colors_state = MOUSE_MINDB;
  else
    w->mouse_colors_state = MOUSE_MAXDB;
  w->mouse_colors_x = x;
  w->mouse_colors_y = y;
  w->mouse_colors_mindB = w->colormap.mindB;
  w->mouse_colors_maxdB = w->colormap.maxdB;
}

void gui_colors_mouse_move(struct mainwin *w, int x, int y)
{
  float delta_dB = (y - w->mouse_colors_y) * -0.1;
  if (w->mouse_colors_state == MOUSE_MINDB) {
    gui_colors_set_mindB(w, w->mouse_colors_mindB + delta_dB);
    gui_state(w);
  } else if (w->mouse_colors_state == MOUSE_MAXDB) {
    gui_colors_set_maxdB(w, w->mouse_colors_maxdB + delta_dB);
    gui_state(w);
  }
}

void gui_colors_mouse_unclick(struct mainwin *w)
{
  w->mouse_colors_state = 0;
}

void gui_fft_size(mainwin *this, int size)
{
  float overlap = this->f.overlap;
  int to_process = this->f.to_process;
  power_buffer_close(&this->f);
  power_buffer_init(&this->f, this->width, size,
                    this->f.buffer_new_line, this->f.bnl_data,
                    this->f.window.type);
  this->redraw_lines(this, 0, this->width-1);
  power_buffer_set_to_process(&this->f, to_process);
  power_buffer_set_overlap(&this->f, overlap);
}

void gui_show_colormap_selection(mainwin *this)
{
  if (this->colmap.shown) return;
  this->colmap.shown = 1;
  init_gui_colormap(&this->colmap, this->colmap.curdir, this);
  gui_widget_show_all(this->colmap.w);
}

void gui_load_colormap(mainwin *this, const char *filename)
{
  colormap m;
  if (!load_colormap(&m, filename, 1)) {
    free_colormap(&this->colormap);
    this->colormap = m;
    redraw_lines_colors(this);
    this->redraw_lines(this, 0, this->width-1);
  }
}
